<?php
// no direct access
defined('_JEXEC') or die('Restricted access');

/* * ***********************************************************************
 * K2 App for Watchful
 * Copyright (c) 2006-2020 Watchful
 * ************************************************************************
 * Watchful.net is a web site monitoring and security service.
 * 
 * Although Watchful.net is a paid service, this program is free software;
 * you can redistribute it and/or modify it under the terms of the GNU 
 * General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 *
 * This header must not be removed. Additional contributions/changes
 * may be added to this header as long as no information is deleted.
 * ************************************************************************
 * Get the latest version of this program at: https://watchful.net
 * *********************************************************************** */

//Require the watchfulliApps class in class.apps.php
require_once(JPATH_ADMINISTRATOR . '/components/com_watchfulli/classes/apps.php');
// We extend a base class to pass additional parameters in the language strings
// It will probably be included in the client but we also put it here so the app will work
// even for older version of the Watchful client.
class K2AppAlert extends AppAlert
{
    public $parameter1;
    public $parameter2;
    public $parameter3;

    public function K2AppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        if ($level != null && $message != null)
        {
            $this->level = $level;
            $this->message = $message;
            $this->parameter1 = $parameter1;
            $this->parameter2 = $parameter2;
            $this->parameter3 = $parameter3;
        }
    }

}
class plgWatchfulliAppsK2App extends watchfulliApps
{

    public function createAppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        $alert = new K2AppAlert($level, $message, $parameter1, $parameter2, $parameter3);
        $this->addAlert($alert);
    }

    public function appMainProgram($oldValuesSerialized = null)
    {
        $this->setName("k2App");
        $this->setDescription("This App checks if the K2 items list has been changed since last check.");
        $debug = $this->params->get('debug', 0);

        $query = $this->buildQuery();

        $db = & JFactory::getDBO();
        $db->setQuery($query);
        if (version_compare(JVERSION, '1.6.0', 'ge'))
        {
            $itemIds = $db->loadColumn();
        }
        else
        {
            $itemIds = $db->loadResultArray();
        }

        if ($debug)
            $this->createAppAlert(1, "[DEBUG][K2APP] Item list: " . print_r($itemIds, true));

        if ($itemIds)
        {
            $newValue = json_encode($itemIds);
        }
        else
        {
            $newValue = "[]";
        }
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][K2APP] New value: " . print_r($newValue, true));

        //Looking for an old value
        $oldValue = $this->readExAppValue($oldValuesSerialized);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][K2APP] Old value: " . print_r($oldValue, true));

        if ($oldValue == null) //First Time the app is triggered
        {
            $this->createAppValue('k2App', $newValue);
        }
        elseif ($oldValue != $newValue)
        {
            $this->createAppAlert(1, "K2APP_LIST_CHANGED");
            $this->updateAppValue('k2App', $newValue);
            $oldItemIds = json_decode($oldValue);
            if (is_array($oldItemIds))
            {
                // check if there are new items
                if (count(array_diff($itemIds, $oldItemIds)))
                {
                    if ($this->params->get('show_titles', 0) == 1)
                    {
                        // Show individual item titles
                        $addedItemIds = array_diff($itemIds, $oldItemIds);
                        $titles = $this->getTitles($addedItemIds);
                        foreach ($addedItemIds as $itemId)
                        {
                            $this->createAppAlert(1, "K2APP_ITEM_ADDED", $titles[$itemId]);
                        }
                    }
                    else
                    {
                        // Just tell that there are new items
                        $this->createAppAlert(1, "K2APP_ITEMS_ADDED");
                    }
                }
                // check if there are deleted items
                if (count(array_diff($oldItemIds, $itemIds)))
                {
                    $this->createAppAlert(1, "K2APP_ITEMS_REMOVED");
                }
            }
        }

        return $this;
    }

    private function buildQuery()
    {
        $query = "SELECT id FROM #__k2_items";

        $where = array();

        // Add states filter
        $states = $this->params->get('states', array());
        if (!is_array($states))
            $states = array($states);
        if (!empty($states))
        {
            $where[] = "published IN (" . implode(',', $states) . ")";
        }

        // Add categories filter
        $categories = $this->params->get('categories', array());
        if (!is_array($categories))
            $categories = array($categories);
        if (!empty($categories))
        {
            $where[] = "catid IN (" . implode(',', $categories) . ")";
        }

        // Set filters
        if (count($where))
        {
            $query .= " WHERE " . implode(" AND ", $where);
        }

        $query .= " ORDER BY id ASC";
        return $query;
    }

    private function getTitles($itemIds)
    {
        $db = & JFactory::getDbo();
        $query = "SELECT id, title FROM #__k2_items WHERE id IN (" . implode(',', $itemIds) . ")";
        $db->setQuery($query);
        foreach ($db->loadObjectList() as $item)
        {
            $titles[$item->id] = $item->title;
        }
        return $titles;
    }

}
